/**
 * @file
 * Ethernet Interface for standalone applications (without RTOS) - works only for
 * ethernet polling mode (polling for ethernet frame reception)
 *
 */

/*
 * Copyright (c) 2001-2004 Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 *
 * Author: Adam Dunkels <adam@sics.se>
 *
 */

#include "ethernetif_eth.h"
#include "lwip/def.h"
#include "lwip/err.h"
#include "lwip/mem.h"
#include "lwip/opt.h"
#include "lwip/pbuf.h"
#include "lwip/timeouts.h"
#include "lwip_init.h"
#include "main.h"
#include "netif/etharp.h"
#include "stm32f4x7_eth.h"
#include <string.h>

#define ENET_RX_TASK_PRIO     (4)
#define ENET_RX_TASK_STK_SIZE (2048)
CPU_STK enet_rx_task_stk[ENET_RX_TASK_STK_SIZE];

/* Network interface name */
#define IFNAME0 's'
#define IFNAME1 't'

/* Ethernet Rx & Tx DMA Descriptors */
extern ETH_DMADESCTypeDef DMARxDscrTab[ETH_RXBUFNB], DMATxDscrTab[ETH_TXBUFNB];

/* Ethernet Driver Receive buffers  */
extern uint8_t Rx_Buff[ETH_RXBUFNB][ETH_RX_BUF_SIZE];

/* Ethernet Driver Transmit buffers */
extern uint8_t Tx_Buff[ETH_TXBUFNB][ETH_TX_BUF_SIZE];

/* Global pointers to track current transmit and receive descriptors */
extern ETH_DMADESCTypeDef *DMATxDescToSet;
extern ETH_DMADESCTypeDef *DMARxDescToGet;

/* Global pointer for last received frame infos */
extern ETH_DMA_Rx_Frame_infos *DMA_RX_FRAME_infos;

static struct netif *low_netif     = NULL;
OS_EVENT            *g_enet_rx_sem = NULL;

static void ethernetif_input(void *pvParameters);
/**
 * In this function, the hardware should be initialized.
 * Called from ethernetif_init().
 *
 * @param netif the already initialized lwip network interface structure
 *        for this ethernetif
 */
static void low_level_init(struct netif *netif)
{
#ifdef CHECKSUM_BY_HARDWARE
    int i;
#endif
    /* set MAC hardware address length */
    netif->hwaddr_len = ETHARP_HWADDR_LEN;

    /* set MAC hardware address */ /* 网卡的 MAC 修改2 */
    netif->hwaddr[0] = ethdev.mac[0];
    netif->hwaddr[1] = ethdev.mac[1];
    netif->hwaddr[2] = ethdev.mac[2];
    netif->hwaddr[3] = ethdev.mac[3];
    netif->hwaddr[4] = ethdev.mac[4];
    netif->hwaddr[5] = ethdev.mac[5];

    /* initialize MAC address in ethernet MAC */
    ETH_MACAddressConfig(ETH_MAC_Address0, netif->hwaddr);

    /* maximum transfer unit */
    netif->mtu = 1500;

    /* device capabilities */
    /* don't set NETIF_FLAG_ETHARP if this device is not an ethernet one */
    netif->flags = NETIF_FLAG_BROADCAST | NETIF_FLAG_ETHARP | NETIF_FLAG_LINK_UP;

    low_netif = netif;

    /* create binary semaphore used for informing ethernetif of frame reception */
    if (NULL == g_enet_rx_sem)
    {
        g_enet_rx_sem = OSSemCreate(0);
    }

    /* Initialize Tx Descriptors list: Chain Mode */
    ETH_DMATxDescChainInit(DMATxDscrTab, &Tx_Buff[0][0], ETH_TXBUFNB);
    /* Initialize Rx Descriptors list: Chain Mode  */
    ETH_DMARxDescChainInit(DMARxDscrTab, &Rx_Buff[0][0], ETH_RXBUFNB);

    /* Enable Ethernet Rx interrrupt */
    for (i = 0; i < ETH_RXBUFNB; i++)
    {
        ETH_DMARxDescReceiveITConfig(&DMARxDscrTab[i], ENABLE);
    }

#ifdef CHECKSUM_BY_HARDWARE
    /* Enable the TCP, UDP and ICMP checksum insertion for the Tx frames */
    for (i = 0; i < ETH_TXBUFNB; i++)
    {
        ETH_DMATxDescChecksumInsertionConfig(&DMATxDscrTab[i], ETH_DMATxDesc_ChecksumTCPUDPICMPFull);
    }
#endif

    /* Note: TCP, UDP, ICMP checksum checking for received frame are enabled in DMA config */

    /* create the task that handles the ENET RX */
    OSTaskCreateExt((void (*)(void *))ethernetif_input,
                    (void *)0,
                    (OS_STK *)&enet_rx_task_stk[ENET_RX_TASK_STK_SIZE - 1],
                    (INT8U)ENET_RX_TASK_PRIO,
                    (INT16U)ENET_RX_TASK_PRIO,
                    (OS_STK *)&enet_rx_task_stk[0],
                    (INT32U)ENET_RX_TASK_STK_SIZE,
                    (void *)0,
                    (INT16U)OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR | OS_TASK_OPT_SAVE_FP);

    /* Enable MAC and DMA transmission and reception */
    ETH_Start();
}

/**
 * This function should do the actual transmission of the packet. The packet is
 * contained in the pbuf that is passed to the function. This pbuf
 * might be chained.
 *
 * @param netif the lwip network interface structure for this ethernetif
 * @param p the MAC packet to send (e.g. IP packet including MAC addresses and type)
 * @return ERR_OK if the packet could be sent
 *         an err_t value if the packet couldn't be sent
 *
 * @note Returning ERR_MEM here if a DMA queue of your MAC is full can lead to
 *       strange results. You might consider waiting for space in the DMA queue
 *       to become availale since the stack doesn't retry to send a packet
 *       dropped because of memory failure (except for the TCP timers).
 */

static err_t low_level_output(struct netif *netif, struct pbuf *p)
{
    static OS_EVENT         *p_enet_tx_sem = NULL;
    INT8U                    err;
    struct pbuf             *q;
    u8                      *buffer;
    __IO ETH_DMADESCTypeDef *DmaTxDesc;
    uint16_t                 framelength     = 0;
    uint32_t                 bufferoffset    = 0;
    uint32_t                 byteslefttocopy = 0;
    uint32_t                 payloadoffset   = 0;

    SYS_ARCH_DECL_PROTECT(sr);

    if (NULL == p_enet_tx_sem)
    {
        p_enet_tx_sem = OSSemCreate(1);
    }

    OSSemPend(p_enet_tx_sem, 0, &err);

    SYS_ARCH_PROTECT(sr);

    DmaTxDesc    = DMATxDescToSet;
    buffer       = (u8 *)(DmaTxDesc->Buffer1Addr);
    bufferoffset = 0;

    for (q = p; q != NULL; q = q->next)
    {
        if ((DmaTxDesc->Status & ETH_DMATxDesc_OWN) != (u32)RESET)
        {
            goto error;
        }

        /* Get bytes in current lwIP buffer  */
        byteslefttocopy = q->len;
        payloadoffset   = 0;

        /* Check if the length of data to copy is bigger than Tx buffer size*/
        while ((byteslefttocopy + bufferoffset) > ETH_TX_BUF_SIZE)
        {
            /* Copy data to Tx buffer*/
            memcpy((u8_t *)((u8_t *)buffer + bufferoffset), (u8_t *)((u8_t *)q->payload + payloadoffset), (ETH_TX_BUF_SIZE - bufferoffset));

            /* Point to next descriptor */
            DmaTxDesc = (ETH_DMADESCTypeDef *)(DmaTxDesc->Buffer2NextDescAddr);

            /* Check if the buffer is available */
            if ((DmaTxDesc->Status & ETH_DMATxDesc_OWN) != (u32)RESET)
            {
                goto error;
            }

            buffer = (u8 *)(DmaTxDesc->Buffer1Addr);

            byteslefttocopy = byteslefttocopy - (ETH_TX_BUF_SIZE - bufferoffset);
            payloadoffset   = payloadoffset + (ETH_TX_BUF_SIZE - bufferoffset);
            framelength     = framelength + (ETH_TX_BUF_SIZE - bufferoffset);
            bufferoffset    = 0;
        }

        /* Copy the remaining bytes */
        memcpy((u8_t *)((u8_t *)buffer + bufferoffset), (u8_t *)((u8_t *)q->payload + payloadoffset), byteslefttocopy);
        bufferoffset = bufferoffset + byteslefttocopy;
        framelength  = framelength + byteslefttocopy;
    }

    /* Prepare transmit descriptors to give to DMA*/
    ETH_Prepare_Transmit_Descriptors(framelength);

/* Give semaphore and exit */
error:

    /* give semaphore and exit */
    OSSemPost(p_enet_tx_sem);

    SYS_ARCH_UNPROTECT(sr);

    return ERR_OK;
}

/**
 * Should allocate a pbuf and transfer the bytes of the incoming
 * packet from the interface into the pbuf.
 *
 * @param netif the lwip network interface structure for this ethernetif
 * @return a pbuf filled with the received packet (including MAC header)
 *         NULL on memory error
 */
static struct pbuf *low_level_input(struct netif *netif)
{
    struct pbuf             *p = NULL, *q;
    u32_t                    len;
    FrameTypeDef             frame;
    u8                      *buffer;
    __IO ETH_DMADESCTypeDef *DMARxDesc;
    uint32_t                 bufferoffset    = 0;
    uint32_t                 payloadoffset   = 0;
    uint32_t                 byteslefttocopy = 0;
    uint32_t                 i               = 0;

    /* get received frame */
    frame = ETH_Get_Received_Frame_interrupt();

    /* Obtain the size of the packet and put it into the "len" variable. */
    len    = frame.length;
    buffer = (u8 *)frame.buffer;

    if (len > 0)
    {
        /* We allocate a pbuf chain of pbufs from the Lwip buffer pool */
        p = pbuf_alloc(PBUF_RAW, len, PBUF_POOL);
    }

    if (p != NULL)
    {
        DMARxDesc    = frame.descriptor;
        bufferoffset = 0;
        for (q = p; q != NULL; q = q->next)
        {
            byteslefttocopy = q->len;
            payloadoffset   = 0;

            /* Check if the length of bytes to copy in current pbuf is bigger than Rx buffer size*/
            while ((byteslefttocopy + bufferoffset) > ETH_RX_BUF_SIZE)
            {
                /* Copy data to pbuf*/
                memcpy((u8_t *)((u8_t *)q->payload + payloadoffset), (u8_t *)((u8_t *)buffer + bufferoffset), (ETH_RX_BUF_SIZE - bufferoffset));

                /* Point to next descriptor */
                DMARxDesc = (ETH_DMADESCTypeDef *)(DMARxDesc->Buffer2NextDescAddr);
                buffer    = (unsigned char *)(DMARxDesc->Buffer1Addr);

                byteslefttocopy = byteslefttocopy - (ETH_RX_BUF_SIZE - bufferoffset);
                payloadoffset   = payloadoffset + (ETH_RX_BUF_SIZE - bufferoffset);
                bufferoffset    = 0;
            }

            /* Copy remaining data in pbuf */
            memcpy((u8_t *)((u8_t *)q->payload + payloadoffset), (u8_t *)((u8_t *)buffer + bufferoffset), byteslefttocopy);
            bufferoffset = bufferoffset + byteslefttocopy;
        }

        /* Release descriptors to DMA */
        DMARxDesc = frame.descriptor;

        /* Set Own bit in Rx descriptors: gives the buffers back to DMA */
        for (i = 0; i < DMA_RX_FRAME_infos->Seg_Count; i++)
        {
            DMARxDesc->Status = ETH_DMARxDesc_OWN;
            DMARxDesc         = (ETH_DMADESCTypeDef *)(DMARxDesc->Buffer2NextDescAddr);
        }

        /* Clear Segment_Count */
        DMA_RX_FRAME_infos->Seg_Count = 0;
        /* added for test*/
    }

    /* When Rx Buffer unavailable flag is set: clear it and resume reception */
    if ((ETH->DMASR & ETH_DMASR_RBUS) != (u32)RESET)
    {
        /* Clear RBUS ETHERNET DMA flag */
        ETH->DMASR = ETH_DMASR_RBUS;
        /* Resume DMA reception */
        ETH->DMARPDR = 0;
    }
    return p;
}

/**
 * This function should be called when a packet is ready to be read
 * from the interface. It uses the function low_level_input() that
 * should handle the actual reception of bytes from the network
 * interface. Then the type of the received packet is determined and
 * the appropriate input function is called.
 *
 * @param netif the lwip network interface structure for this ethernetif
 */
void ethernetif_input(void *pvParameters)
{
    struct pbuf *p;
    INT8U        err;

    SYS_ARCH_DECL_PROTECT(sr);

    for (;;)
    {

        OSSemPend(g_enet_rx_sem, 0, &err);

TRY_GET_NEXT_FRAME:
        SYS_ARCH_PROTECT(sr);
        p = low_level_input(low_netif);
        SYS_ARCH_UNPROTECT(sr);

        if (p != NULL)
        {
            if (ERR_OK != low_netif->input(p, low_netif))
            {
                pbuf_free(p);
            }
            else
            {
                goto TRY_GET_NEXT_FRAME;
            }
        }
    }
}

/**
 * Should be called at the beginning of the program to set up the
 * network interface. It calls the function low_level_init() to do the
 * actual setup of the hardware.
 *
 * This function should be passed as a parameter to netif_add().
 *
 * @param netif the lwip network interface structure for this ethernetif
 * @return ERR_OK if the loopif is initialized
 *         ERR_MEM if private data couldn't be allocated
 *         any other err_t on error
 */
err_t ethernetif_eth_init(struct netif *netif)
{
    LWIP_ASSERT("netif != NULL", (netif != NULL));

#if LWIP_NETIF_HOSTNAME
    /* Initialize interface hostname */
    netif->hostname = "lwip";
#endif /* LWIP_NETIF_HOSTNAME */

    netif->name[0] = IFNAME0;
    netif->name[1] = IFNAME1;

    netif->output     = etharp_output;
    netif->linkoutput = low_level_output;

    /* initialize the hardware */
    low_level_init(netif);

    return ERR_OK;
}